<?php
namespace Ep\Gath\DataStore;

use Ep\Gath\Plugin\Products;
use WC_Product;
use WC_Product_Variable_Data_Store_CPT;

/**
 * Class VariableDataStore
 *
 * @package Ep\Gath\DataStore
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class VariableDataStore extends WC_Product_Variable_Data_Store_CPT
{
    /**
     * Read attributes from post meta.
     *
     * @param WC_Product $product Product object.
     */
    
    protected function read_attributes(&$product)
    {
        if ($product->get_id() < Products::ID_OFFSET) {
            parent::read_attributes($product);
        }

        /** @var Products $gath */
        $gath = apply_filters('ep_gath_products', null);

        if (!$gath) {
            parent::read_attributes($product);
        }

        $result = $gath->getRawResult($product->get_id());
        $attributes_key = array_key_exists('VarAttributes', $result) ? 'VarAttributes' : 'Attributes';

        if (isset($result[$attributes_key]) && is_array($result[$attributes_key]) && !empty($result[$attributes_key])) {
            /** @var \WC_Product_Attribute[] $attributes */
            $attributes = [];

            foreach ($result[$attributes_key] as $i => $attr) {
                $id = $attr['id'];

                if (!isset($attributes[$id])) {
                    $attribute = new \WC_Product_Attribute();
                    $attribute->set_id(0);
                    $attribute->set_name($attr['name']);
                    $attribute->set_options([$attr['value']]);
                    $attribute->set_position($i + 1);
                    $attribute->set_visible(1);
                    $attribute->set_variation($product->get_type() === 'variable' ? 1 : 0);
                    $attributes[$id] = $attribute;
                } else {
                    $options = $attributes[$id]->get_options();
                    $options[] = $attr['value'];
                    $attributes[$id]->set_options($options);
                }
            }
            $product->set_attributes(array_values($attributes));
        }
    }

    /**
     * Get an array of all sale and regular prices from all variations. This is used for example when displaying the price range at variable product level or seeing if the variable product is on sale.
     *
     * Can be filtered by plugins which modify costs, but otherwise will include the raw meta costs unlike get_price() which runs costs through the woocommerce_get_price filter.
     * This is to ensure modified prices are not cached, unless intended.
     *
     * @since  3.0.0
     * @param  WC_Product $product Product object.
     * @param  bool       $for_display If true, prices will be adapted for display based on the `woocommerce_tax_display_shop` setting (including or excluding taxes).
     * @return array of prices
     */
    public function read_price_data( &$product, $for_display = false ) {

        /**
         * Transient name for storing prices for this product (note: Max transient length is 45)
         *
         * @since 2.5.0 a single transient is used per product for all prices, rather than many transients per product.
         */
        $transient_name = 'wc_var_prices_' . $product->get_id();

        $price_hash = $this->get_price_hash( $product, $for_display );

        /**
         * $this->prices_array is an array of values which may have been modified from what is stored in transients - this may not match $transient_cached_prices_array.
         * If the value has already been generated, we don't need to grab the values again so just return them. They are already filtered.
         */
        if ( empty( $this->prices_array[ $price_hash ] ) ) {
            $transient_cached_prices_array = ['version' => null];

            // If the product version has changed since the transient was last saved, reset the transient cache.
            if ( empty( $transient_cached_prices_array['version'] ) || \WC_Cache_Helper::get_transient_version( 'product' ) !== $transient_cached_prices_array['version'] ) {
                $transient_cached_prices_array = array( 'version' => \WC_Cache_Helper::get_transient_version( 'product' ) );
            }

            // If the prices are not stored for this hash, generate them and add to the transient.
            if ( empty( $transient_cached_prices_array[ $price_hash ] ) ) {
                $prices         = array();
                $regular_prices = array();
                $sale_prices    = array();
                $variation_ids  = $product->get_visible_children();
                foreach ( $variation_ids as $variation_id ) {
                    if ( $variation = wc_get_product( $variation_id ) ) {
                        $price         = apply_filters( 'woocommerce_variation_prices_price', $variation->get_price( 'edit' ), $variation, $product );
                        $regular_price = apply_filters( 'woocommerce_variation_prices_regular_price', $variation->get_regular_price( 'edit' ), $variation, $product );
                        $sale_price    = apply_filters( 'woocommerce_variation_prices_sale_price', $variation->get_sale_price( 'edit' ), $variation, $product );

                        // Skip empty prices.
                        if ( '' === $price ) {
                            continue;
                        }

                        // If sale price does not equal price, the product is not yet on sale.
                        if ( $sale_price === $regular_price || $sale_price !== $price ) {
                            $sale_price = $regular_price;
                        }

                        // If we are getting prices for display, we need to account for taxes.
                        if ( $for_display ) {
                            if ( 'incl' === get_option( 'woocommerce_tax_display_shop' ) ) {
                                $price         = '' === $price ? ''         : wc_get_price_including_tax( $variation, array( 'qty' => 1, 'price' => $price ) );
                                $regular_price = '' === $regular_price ? '' : wc_get_price_including_tax( $variation, array( 'qty' => 1, 'price' => $regular_price ) );
                                $sale_price    = '' === $sale_price ? ''    : wc_get_price_including_tax( $variation, array( 'qty' => 1, 'price' => $sale_price ) );
                            } else {
                                $price         = '' === $price ? ''         : wc_get_price_excluding_tax( $variation, array( 'qty' => 1, 'price' => $price ) );
                                $regular_price = '' === $regular_price ? '' : wc_get_price_excluding_tax( $variation, array( 'qty' => 1, 'price' => $regular_price ) );
                                $sale_price    = '' === $sale_price ? ''    : wc_get_price_excluding_tax( $variation, array( 'qty' => 1, 'price' => $sale_price ) );
                            }
                        }

                        $prices[ $variation_id ]         = wc_format_decimal( $price, wc_get_price_decimals() );
                        $regular_prices[ $variation_id ] = wc_format_decimal( $regular_price, wc_get_price_decimals() );
                        $sale_prices[ $variation_id ]    = wc_format_decimal( $sale_price . '.00', wc_get_price_decimals() );
                    }
                }

                $transient_cached_prices_array[ $price_hash ] = array(
                    'price'         => $prices,
                    'regular_price' => $regular_prices,
                    'sale_price'    => $sale_prices,
                );

                //set_transient( $transient_name, wp_json_encode( $transient_cached_prices_array ), DAY_IN_SECONDS * 30 );
            }

            /**
             * Give plugins one last chance to filter the variation prices array which has been generated and store locally to the class.
             * This value may differ from the transient cache. It is filtered once before storing locally.
             */
            $this->prices_array[ $price_hash ] = apply_filters( 'woocommerce_variation_prices', $transient_cached_prices_array[ $price_hash ], $product, $for_display );
        }
        return $this->prices_array[ $price_hash ];
    }
}
